@@ -1,23 +1,14 @@
 package ZMQ::FFI;
-$ZMQ::FFI::VERSION = '1.16_01'; # TRIAL
-
-$ZMQ::FFI::VERSION = '1.1601';# ABSTRACT: version agnostic Perl bindings for zeromq using ffi
+$ZMQ::FFI::VERSION = '1.01';
+# ABSTRACT: version agnostic Perl bindings for zeromq using ffi
 
 use strict;
 use warnings;
 
 use ZMQ::FFI::Util qw(zmq_soname zmq_version valid_soname);
+use ZMQ::FFI::ErrorHelper;
 use Carp;
 
-use Import::Into;
-
-sub import {
-    my ($pkg, @import_args) = @_;
-
-    my $target = caller;
-    ZMQ::FFI::Constants->import::into($target, @import_args);
-}
-
 sub new {
     my ($self, %args) = @_;
 
@@ -30,27 +21,23 @@ sub new {
         $args{soname} = zmq_soname( die => 1 );
     }
 
-    my ($major, $minor) = zmq_version($args{soname});
+    # explicitly passing in a loaded error helper instance (i.e. zmq error
+    # bindings) guards against the OS X loader clobbering errno, which can
+    # happen if the bindings are loaded lazily
+    $args{error_helper} =
+        ZMQ::FFI::ErrorHelper->new( soname => $args{soname} );
+
+    my ($major) = zmq_version($args{soname});
 
     if ($major == 2) {
         require ZMQ::FFI::ZMQ2::Context;
         return ZMQ::FFI::ZMQ2::Context->new(%args);
     }
-    elsif ($major == 3) {
+    else {
         require ZMQ::FFI::ZMQ3::Context;
         return ZMQ::FFI::ZMQ3::Context->new(%args);
-    } 
-    else {
-	if ($major == 4 and $minor == 0) {
-            require ZMQ::FFI::ZMQ4::Context;
-            return ZMQ::FFI::ZMQ4::Context->new(%args);
-        }
-        else {
-            require ZMQ::FFI::ZMQ4_1::Context;
-            return ZMQ::FFI::ZMQ4_1::Context->new(%args);
-        }
     }
-}
+};
 
 1;
 
@@ -66,17 +53,18 @@ ZMQ::FFI - version agnostic Perl bindings for zeromq using ffi
 
 =head1 VERSION
 
-version 1.16_01
+version 1.01
 
 =head1 SYNOPSIS
 
     #### send/recv ####
 
     use v5.10;
-    use ZMQ::FFI qw(ZMQ_REQ ZMQ_REP);
+    use ZMQ::FFI;
+    use ZMQ::FFI::Constants qw(ZMQ_REQ ZMQ_REP);
 
     my $endpoint = "ipc://zmq-ffi-$$";
-    my $ctx      = ZMQ::FFI->new();
+    my $ctx      = ZMQ::FFI->new( threads => 1 );
 
     my $s1 = $ctx->socket(ZMQ_REQ);
     $s1->connect($endpoint);
@@ -93,7 +81,8 @@ version 1.16_01
     #### pub/sub ####
 
     use v5.10;
-    use ZMQ::FFI qw(ZMQ_PUB ZMQ_SUB);
+    use ZMQ::FFI;
+    use ZMQ::FFI::Constants qw(ZMQ_PUB ZMQ_SUB);
     use Time::HiRes q(usleep);
 
     my $endpoint = "ipc://zmq-ffi-$$";
@@ -143,7 +132,8 @@ version 1.16_01
     #### multipart ####
 
     use v5.10;
-    use ZMQ::FFI qw(ZMQ_DEALER ZMQ_ROUTER);
+    use ZMQ::FFI;
+    use ZMQ::FFI::Constants qw(ZMQ_DEALER ZMQ_ROUTER);
 
     my $endpoint = "ipc://zmq-ffi-$$";
     my $ctx      = ZMQ::FFI->new();
@@ -165,7 +155,8 @@ version 1.16_01
     #### nonblocking ####
 
     use v5.10;
-    use ZMQ::FFI qw(ZMQ_PUSH ZMQ_PULL);
+    use ZMQ::FFI;
+    use ZMQ::FFI::Constants qw(ZMQ_PUSH ZMQ_PULL);
     use AnyEvent;
     use EV;
 
@@ -187,7 +178,7 @@ version 1.16_01
 
             $recv++;
             if ($recv == 3) {
-                EV::break();
+                EV::unloop();
             }
         }
     };
@@ -235,15 +226,18 @@ newer.
 
 =head1 CONTEXT API
 
-=head2 new
+=head2 new([threads, max_sockets, soname])
 
-    my $ctx = ZMQ::FFI->new(%options);
+    ZMQ::FFI->new()
+
+    ZMQ::FFI->new( threads => 42, max_sockets => 42 )
+
+    ZMQ::FFI->new( soname => '/path/to/libzmq.so' )
+    ZMQ::FFI->new( soname => 'libzmq.so.3' )
 
 returns a new context object, appropriate for the version of
 libzmq found on your system. It accepts the following optional attributes:
 
-=head3 options
-
 =over 4
 
 =item threads
@@ -258,9 +252,6 @@ max number of sockets allowed for context. Default: 1024
 
 =item soname
 
-    ZMQ::FFI->new( soname => '/path/to/libzmq.so' );
-    ZMQ::FFI->new( soname => 'libzmq.so.3' );
-
 specify the libzmq library name to load.  By default ZMQ::FFI will first try
 the generic soname for the system, then the soname for each version of zeromq
 (e.g. libzmq.so.3). C<soname> can also be the path to a particular libzmq so
@@ -271,21 +262,19 @@ the same process, though the utility of doing such a thing is dubious
 
 =back
 
-=head2 version
-
-    my ($major, $minor, $patch) = $ctx->version();
+=head2 ($major, $minor, $patch) = version()
 
 return the libzmq version as the list C<($major, $minor, $patch)>
 
-=head2 get
+=head2 get($option)
 
 I<requires zmq E<gt>= 3.x>
 
-    my $threads = $ctx->get(ZMQ_IO_THREADS)
+    $ctx->get(ZMQ_IO_THREADS)
 
 get a context option value
 
-=head2 set
+=head2 set($option, $option_value)
 
 I<requires zmq E<gt>= 3.x>
 
@@ -293,36 +282,28 @@ I<requires zmq E<gt>= 3.x>
 
 set a context option value
 
-=head2 socket
+=head2 $socket = socket($type)
 
-    my $socket = $ctx->socket(ZMQ_REQ)
+    $ctx->socket(ZMQ_REQ)
 
-returns a socket of the specified type. See L</SOCKET API> below
+returns a socket of the specified type. See L<SOCKET API> below
 
-=head2 proxy
-
-    $ctx->proxy($frontend, $backend);
-
-    $ctx->proxy($frontend, $backend, $capture);
+=head2 proxy($frontend, $backend, [$capture])
 
 sets up and runs a C<zmq_proxy>. For zmq 2.x this will use a C<ZMQ_STREAMER>
 device to simulate the proxy. The optional C<$capture> is only supported for
 zmq E<gt>= 3.x however
 
-=head2 device
+=head2 device($type, $frontend, $backend)
 
 I<zmq 2.x only>
 
-    $ctx->device($type, $frontend, $backend);
-
 sets up and runs a C<zmq_device> with specified frontend and backend sockets
 
-=head2 destroy
+=head2 destroy()
 
-destroy the underlying zmq context. In general you shouldn't have to call this
-directly as it is called automatically for you when the object gets reaped
-
-See L</CLEANUP> below
+destroys the underlying zmq context. This is called automatically when the
+object gets reaped
 
 =head1 SOCKET API
 
@@ -335,261 +316,110 @@ provided that will work independent of version.
 As attributes are constantly being added/removed from zeromq, it is unlikely
 the 'static' accessors will grow much beyond the current set.
 
-=head2 version
+=head2 ($major, $minor, $patch) = version()
 
-    my ($major, $minor, $patch) = $socket->version();
+same as Context version() above
 
-same as Context C<version> above
-
-=head2 connect
-
-    $socket->connect($endpoint);
+=head2 connect($endpoint)
 
 does socket connect on the specified endpoint
 
-=head2 disconnect
+=head2 disconnect($endpoint)
 
 I<requires zmq E<gt>= 3.x>
 
-    $socket->disconnect($endpoint);
-
 does socket disconnect on the specified endpoint
 
-=head2 bind
-
-    $socket->bind($endpoint);
+=head2 bind($endpoint)
 
 does socket bind on the specified endpoint
 
-=head2 unbind
+=head2 unbind($endpoint)
 
 I<requires zmq E<gt>= 3.x>
 
-    $socket->unbind($endpoint);
-
 does socket unbind on the specified endpoint
 
-=head2 get_linger, set_linger
+=head2 get_linger(), set_linger($millis)
 
-    my $linger = $socket->get_linger();
+get or set the current socket linger period
 
-    $socket->set_linger($millis);
-
-get or set the socket linger period. Default: 0 (no linger)
-
-See L</CLEANUP> below
-
-=head2 get_identity, set_identity
-
-    my $ident = $socket->get_identity();
-
-    $socket->set_identity($ident);
+=head2 get_identity(), set_identity($ident)
 
 get or set the socket identity for request/reply patterns
 
-=head2 get_fd
-
-    my $fd = $socket->get_fd();
+=head2 get_fd()
 
 get the file descriptor associated with the socket
 
-=head2 get
+=head2 get($option, $option_type)
 
-    my $option_value = $socket->get($option_name, $option_type);
+    $socket->get(ZMQ_LINGER, 'int')
 
-    my $linger = $socket->get(ZMQ_LINGER, 'int');
+return the value for the specified socket option. C<$option_type> is the type
+associated with the option value in the zeromq API (C<zmq_getsockopt> man page)
 
-generic method to get the value for any socket option. C<$option_type> is the
-type associated with C<$option_value> in the zeromq API (C<zmq_getsockopt> man
-page)
+=head2 set($option, $option_type, $option_value)
 
-=head2 set
+    $socket->set(ZMQ_IDENTITY, 'binary', 'foo')
 
-    $socket->set($option_name, $option_type, $option_value);
+set the socket option to the specified value. C<$option_type> is the type
+associated with the option value in the zeromq API (C<zmq_setsockopt> man page)
 
-    $socket->set(ZMQ_IDENTITY, 'binary', 'foo');
-
-generic method to set the value for any socket option.  C<$option_type> is the
-type associated with C<$option_value> in the zeromq API (C<zmq_setsockopt> man
-page)
-
-=head2 subscribe
-
-    $socket->subscribe($topic);
+=head2 subscribe($topic)
 
 add C<$topic> to the subscription list
 
-=head2 unsubscribe
-
-    $socket->unsubscribe($topic);
+=head2 unsubscribe($topic)
 
 remove C<$topic> from the subscription list
 
-=head2 send
+=head2 send($msg, [$flags])
 
-    $socket->send($msg);
+    $socket->send('ohhai')
 
-    $socket->send($msg, $flags);
+    $socket->send('ohhai', ZMQ_DONTWAIT)
 
 sends a message using the optional flags
 
-=head2 send_multipart
+=head2 send_multipart($parts_aref, [$flags])
 
-    $socket->send($parts_aref);
-
-    $socket->send($parts_aref, $flags);
+    $socket->send([qw(foo bar baz)])
 
 given an array ref of message parts, sends the multipart message using the
 optional flags. ZMQ_SNDMORE semantics are handled for you
 
-=head2 recv
+=head2 $msg = recv([$flags])
 
-    my $msg = $socket->recv();
+    $socket->recv()
 
-    my $msg = $socket->recv($flags);
+    $socket->recv(ZMQ_DONTWAIT)
 
 receives a message using the optional flags
 
-=head2 recv_multipart
-
-    my @parts = $socket->recv_multipart();
-
-    my @parts = $socket->recv_multipart($flags);
+=head2 @parts = recv_multipart([$flags])
 
 receives a multipart message, returning an array of parts. ZMQ_RCVMORE
 semantics are handled for you
 
 =head2 has_pollin, has_pollout
 
-    while ( $socket->has_pollin ) { ... }
-
 checks ZMQ_EVENTS for ZMQ_POLLIN and ZMQ_POLLOUT respectively, and returns
 true/false depending on the state
 
-=head2 close
+=head2 close()
 
-close the underlying zmq socket. In general you shouldn't have to call this
-directly as it is called automatically for you when the object gets reaped
-
-See L</CLEANUP> below
-
-=head2 die_on_error
-
-    $socket->die_on_error(0);
-
-    $socket->die_on_error(1);
-
-controls whether error handling should be exceptional or not. This is set to
-true by default. See L</ERROR HANDLING> below
-
-=head2 has_error
-
-returns true or false depending on whether the last socket operation had an
-error. This is really just an alias for C<last_errno>
-
-=head2 last_errno
-
-returns the system C<errno> set by the last socket operation, or 0 if there
-was no error
-
-=head2 last_strerror
-
-returns the human readable system error message associated with the socket
-C<last_errno>
-
-=head1 CLEANUP
-
-With respect to cleanup C<ZMQ::FFI> follows either the L<zeromq guide|http://zguide.zeromq.org/page:all#Making-a-Clean-Exit>
-recommendations or the behavior of other zmq bindings.
-That is:
-
-=over 4
-
-=item * it uses 0 linger by default (this is the default used by L<czmq|https://github.com/zeromq/czmq> and L<jzmq|https://github.com/zeromq/jzmq>)
-
-=item * during object destruction it will call close/destroy for you
-
-=item * it arranges the reference hierarchy such that sockets will be properly
-      cleaned up before their associated contexts
-
-=item * it detects fork/thread situations and ensures sockets/contexts are only
-      cleaned up in their originating process/thread
-
-=item * it guards against double closes/destroys
-
-=back
-
-Given the above you're probably better off letting C<ZMQ::FFI> handle cleanup
-for you. But if for some reason you want to do explicit cleanup yourself you
-can. All the below will accomplish the same thing:
-
-    # implicit cleanup
-    {
-        my $context = ZMQ::FFI->new();
-        my $socket  = $ctx->socket($type);
-        ...
-        # close/destroy called in destructors at end of scope
-    }
-
-    # explicit cleanup
-    $socket->close();
-    $context->destroy();
-
-    # ditto
-    undef $socket;
-    undef $context;
-
-Regarding C<linger>, you can always set this to a value you prefer if
-you don't like the default. Once set the new value will be used when the
-socket is subsequently closed (either implicitly or explicitly):
-
-    $socket->set_linger(-1); # infinite linger
-                             # $context->destroy will block forever
-                             # (or until all pending messages have been sent)
+close the underlying zmq socket. This is called automatically when the object
+gets reaped
 
 =head1 ERROR HANDLING
 
-By default, ZMQ::FFI checks the return codes of underlying zmq functions for
-you, and in the case of an error it will die with the human readable system
-error message.
+ZMQ::FFI checks the return codes of underlying zmq functions for you, and in
+the case of an error it will die with the plain english system error message.
 
     $ctx->socket(-1);
     # dies with 'zmq_socket: Invalid argument'
 
-Usually this is what you want, but not always. Some zmq operations can return
-errors that are not fatal and should be handled. For example using
-C<ZMQ_DONTWAIT> with send/recv can return C<EAGAIN> and simply means try
-again, not die.
-
-For situations such as this you can turn off exceptional error handling by
-setting C<die_on_error> to 0. It is then for you to check and manage any zmq
-errors by checking C<last_errno>:
-
-    use Errno qw(EAGAIN);
-
-    my $ctx = ZMQ::FFI->new();
-    my $s   = $ctx->socket(ZMQ_DEALER);
-    $s->bind('tcp://*:7200');
-
-    $s->die_on_error(0); # turn off exceptional error handling
-
-    while (1) {
-        my $msg = $s->recv(ZMQ_DONTWAIT);
-
-        if ($s->last_errno == EAGAIN) {
-            sleep 1;
-        }
-        elsif ($s->last_errno) {
-            die $s->last_strerror;
-        }
-        else {
-            warn "recvd: $msg";
-            last;
-        }
-    }
-
-    $s->die_on_error(1); # turn back on exceptional error handling
-
 =head1 FFI VS XS PERFORMANCE
 
 ZMQ::FFI uses L<FFI::Platypus> on the backend. In addition to a friendly,
@@ -617,13 +447,6 @@ convenience, flexibility, and portability of ffi.
 You can find the detailed performance results that informed this section at:
 L<https://gist.github.com/calid/17df5bcfb81c83786d6f>
 
-=head1 BUGS
-
-C<ZMQ::FFI> is free as in beer in addition to being free as in speech. While
-I've done my best to ensure it's tasty, high quality beer, it probably isn't perfect.
-If you encounter problems, or otherwise see room for improvement, please open
-an issue (or even better a pull request!) on L<github|https://github.com/calid/zmq-ffi>
-
 =head1 SEE ALSO
 
 =over 4
@@ -656,7 +479,7 @@ Dylan Cali <calid1984@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2019 by Dylan Cali.
+This software is copyright (c) 2015 by Dylan Cali.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
