@@ -1,28 +1,13 @@
 package ZMQ::FFI::ContextRole;
-$ZMQ::FFI::ContextRole::VERSION = '1.16_01'; # TRIAL
+$ZMQ::FFI::ContextRole::VERSION = '1.01';
+use Moo::Role;
 
-$ZMQ::FFI::ContextRole::VERSION = '1.1601';use Moo::Role;
-
-use ZMQ::FFI::Util qw(current_tid);
-
-# real underlying zmq context pointer
-has context_ptr => (
+# real underlying zmq ctx pointer
+has _ctx => (
     is      => 'rw',
     default => -1,
 );
 
-# used to make sure we handle fork situations correctly
-has _pid => (
-    is      => 'ro',
-    default => sub { $$ },
-);
-
-# used to make sure we handle thread situations correctly
-has _tid => (
-    is      => 'ro',
-    default => sub { current_tid() },
-);
-
 has soname => (
     is       => 'ro',
     required => 1,
@@ -38,22 +23,13 @@ has max_sockets => (
     predicate => 'has_max_sockets',
 );
 
-has sockets => (
-    is        => 'rw',
-    lazy      => 1,
-    default   => sub { {} },
-);
-
 requires qw(
-    init
     get
     set
     socket
     proxy
     device
     destroy
-    curve_keypair
-    has_capability
 );
 
 1;
@@ -70,7 +46,7 @@ ZMQ::FFI::ContextRole
 
 =head1 VERSION
 
-version 1.16_01
+version 1.01
 
 =head1 AUTHOR
 
@@ -78,7 +54,7 @@ Dylan Cali <calid1984@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2019 by Dylan Cali.
+This software is copyright (c) 2015 by Dylan Cali.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
