@@ -1,11 +1,6 @@
 package ZMQ::FFI::SocketRole;
-$ZMQ::FFI::SocketRole::VERSION = '1.16_01'; # TRIAL
-
-$ZMQ::FFI::SocketRole::VERSION = '1.1601';use FFI::Platypus;
-use FFI::Platypus::Memory qw(malloc);
-
-use ZMQ::FFI::Constants qw(zmq_msg_t_size);
-use ZMQ::FFI::Util qw(current_tid);
+$ZMQ::FFI::SocketRole::VERSION = '1.01';
+use FFI::Platypus;
 
 use Moo::Role;
 
@@ -14,6 +9,13 @@ has soname => (
     required => 1,
 );
 
+# context to associate socket instance with.
+# reference necessary to guard against premature object destruction
+has ctx => (
+    is       => 'ro',
+    required => 1,
+);
+
 # zmq constant socket type, e.g. ZMQ_REQ
 has type => (
     is       => 'ro',
@@ -21,57 +23,17 @@ has type => (
 );
 
 # real underlying zmq socket pointer
-has socket_ptr => (
+has _socket => (
     is      => 'rw',
     default => -1,
 );
 
-# a weak reference to the context object
-has context => (
-    is       => 'ro',
-    required => 1,
-    weak_ref => 1,
-);
-
-# message struct to reuse when sending/receiving
-has _zmq_msg_t => (
-    is        => 'ro',
-    lazy      => 1,
-    builder   => '_build_zmq_msg_t',
-);
-
-# used to make sure we handle fork situations correctly
-has _pid => (
-    is      => 'ro',
-    default => sub { $$ },
-);
-
-# used to make sure we handle thread situations correctly
-has _tid => (
-    is      => 'ro',
-    default => sub { current_tid() },
-);
-
 has sockopt_sizes => (
     is      => 'ro',
     lazy    => 1,
     builder => '_build_sockopt_sizes'
 );
 
-sub _build_zmq_msg_t {
-    my ($self) = @_;
-
-    my $msg_ptr;
-    {
-        no strict q/refs/;
-        my $class = ref $self;
-        $msg_ptr = malloc(zmq_msg_t_size);
-        &{"$class\::zmq_msg_init"}($msg_ptr);
-    }
-
-    return $msg_ptr;
-}
-
 sub _build_sockopt_sizes {
     my $ffi = FFI::Platypus->new();
 
@@ -119,7 +81,7 @@ ZMQ::FFI::SocketRole
 
 =head1 VERSION
 
-version 1.16_01
+version 1.01
 
 =head1 AUTHOR
 
@@ -127,7 +89,7 @@ Dylan Cali <calid1984@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2019 by Dylan Cali.
+This software is copyright (c) 2015 by Dylan Cali.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
