@@ -1,7 +1,6 @@
 package ZMQ::FFI::Versioner;
-$ZMQ::FFI::Versioner::VERSION = '1.16_01'; # TRIAL
-
-$ZMQ::FFI::Versioner::VERSION = '1.1601';use Moo::Role;
+$ZMQ::FFI::Versioner::VERSION = '1.01';
+use Moo::Role;
 
 use ZMQ::FFI::Util qw(zmq_version);
 
@@ -35,7 +34,7 @@ ZMQ::FFI::Versioner
 
 =head1 VERSION
 
-version 1.16_01
+version 1.01
 
 =head1 AUTHOR
 
@@ -43,7 +42,7 @@ Dylan Cali <calid1984@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2019 by Dylan Cali.
+This software is copyright (c) 2015 by Dylan Cali.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
